#!/usr/bin/perl -w

# ------------------------------------------------------------------------------------
#  Program for downloading XPlanet cloud images from a random mirror
# 
#  Copyright (c) 2003, cueSim Ltd.                  http://www.cueSim.com, Bedford, UK
#  
# ------------------------------------------------------------------------------------
#  
#  Redistribution and use, with or without modification, are permitted provided 
#  that the following conditions are met:
#  
#      * Redistributions of source code must retain the above copyright notice, 
#          this list of conditions and the following disclaimer.
#      * Neither the cueSim name nor the names of its contributors may 
#          be used to endorse or promote products derived from this software without 
#          specific prior written permission.
#  
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
#  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
#  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
#  SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
#  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
#  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# ADDED:
# VerifyImage by Markus Schmidt <schmidt@boomshop.net> in 2010
# Function to check if the image was downloaded correctly. uses Image::Magick
# to check the color of the last 100 pixels. If they're all the same, it is
# declared as corrupt.

use LWP::Simple;
use open IO => ':raw';

# Set options:
#  - where to save the downloaded file (default is current directory)
my $Filename = "clouds.jpg";

# - how often the image is updated on the server
my $MaxDownloadFrequencyHours = 3;

# - how many times to retry, if the server doesn't respond
my $ListTries = 10;

# - how many times to retry, if the server doesn't respond
my $CoralTries = 5;

my $MaxRetries = $ListTries + $CoralTries;

## Note: excessive requests to a single image server is discouraged.
## This script limits max retries, does not download more frequently
## than every two hours (the file is generated every 3 hours). and 
## picks a random mirror location for every download.
##
## Change these settings at the risk of being blacklisted from the
## image servers.

my $GOT_IM = eval "use Image::Magick; 1" ? 1 : 0;
if($GOT_IM) {
    require Image::Magick;
    Image::Magick->import();
}

# Get file details
if(-f $Filename) {
    my @Stats = stat($Filename);
    my $FileAge = (time() - $Stats[9]);
    my $FileSize = $Stats[7];
    # Check if file is already up to date
    if($FileAge < 60 * 60 * $MaxDownloadFrequencyHours && $FileSize > 200000) {
        print "Clouds image is up-to-date\n";
        exit(1);
    }
}


# Try several times to download the file if necessary
for(my $i = 0; $i < $MaxRetries; $i++) {
    # Get a random website to hit for the file
    my $MirrorURL;
    if($i > $ListTries) {
        $MirrorURL = "http://xplanet.sourceforge.net/clouds/clouds_2048.jpg";
        print "Trying CORAL-Network (" . ($i - $ListTries + 1) . "/$CoralTries)...\n";
    } else {
        $MirrorURL = GetRandomMirror();
        print "Trying \"$MirrorURL\"...\n";
    }

    # Download the file
    
    my $Response = getstore($MirrorURL, $Filename);
    
    # If successfully downloaded, try to verify the image
    if( IndicatesSuccess($Response)) {
        print "Transfer completed, file saved to \"$Filename\".\n";
        if($GOT_IM) {
            print "Verifying image data...\n";
            if(verifyImage($Filename)) {
                # File seems to be okay
                print "Image \"$Filename\" - seems to be downloaded successfully.\n";
                exit(0);
            } else {
                # File seems to be corrupt, retrying
                print "Verification of \"$Filename\" failed! Trying another mirror...\n";
            }
        } else {
            print "No PERL-Module Image::Magick found, verification of download not possible.\n";
            exit(0);
        }
    } else {
        # Warning that we're retrying another random server
        print "Download not available, trying another mirror...\n";
    }
}

# Warning that no servers could be contacted
print "ERROR: Tried to download the file $MaxRetries times, but no servers could provide the file.\n";
exit(2);



# Return codes of 200 to 299 are "success" in HTTP-speak
sub IndicatesSuccess
{
    my $Response = shift();
    if($Response =~ /2\d\d/) {
        return(1);
    } else {
        return(0);
    }
}

# Returns the name of an internet resource which can provide the clouds image
sub GetRandomMirror
{
    # Populate a list of mirrors
    my @Mirrors = (
               "http://home.megapass.co.kr/~gitto88/cloud_data/clouds_2048.jpg",
               "http://userpage.fu-berlin.de/~jml/clouds_2048.jpg",
               "http://user.chol.com/~winxplanet/cloud_data/clouds_2048.jpg",
               "http://www.ruwenzori.net/earth/clouds_2048.jpg",
               "http://home.megapass.co.kr/~holywatr/cloud_data/clouds_2048.jpg",
               "http://www.wizabit.eclipse.co.uk/xplanet/files/mirror/clouds_2048.jpg",
               "ftp://ftp.iastate.edu/pub/xplanet/clouds_2048.jpg",
               "http://xplanet.explore-the-world.net/clouds_2048.jpg");
    
    # Return one at random
    return $Mirrors[rand scalar(@Mirrors)];
}

# Returns 0 if the image was not downloaded correctly, 1 in the other case.
# It checks the image via Image::Magick by calling the last 100 pixels of the
# last row checking their color values. If they're all @ 32896 the image is
# supposed to be corrupt.
sub verifyImage
{
    if ($GOT_IM) {
        my ($im) = @_;
        my $image=Image::Magick->new;
        $image->Read($im);
        my ($w, $h) = $image->Get('width', 'height');
        for $x (($w-100)..$w) {
            # walk through 100 of the most-right pixels in the last row
            my ($r, $g, $b) = split(/,/, $image->Get('pixel[' . $x . ',' . $h . ']'));
            if($r + $g + $b < 98688) {
                # if the summarized color is lower than 98688 return success
                return(1);
            }
        }
        return(0);
    } else {
        # no Image::Magick available, return exit code 2
        return(2);
    }
}

